@@ -29,11 +29,11 @@ import re
 from .util import u, unicod, U_EMPTY_STRING, U_SPACE
 from .unicode_util import digit as unicode_digit
 from .re_util import fullmatch
-from .phonemetadata import PhoneMetadata
-from .phonenumberutil import _VALID_PUNCTUATION, REGION_CODE_FOR_NON_GEO_ENTITY
+from .phonemetadata import PhoneMetadata, REGION_CODE_FOR_NON_GEO_ENTITY
+from .phonenumberutil import _VALID_PUNCTUATION
 from .phonenumberutil import _PLUS_SIGN, _PLUS_CHARS_PATTERN
 from .phonenumberutil import _extract_country_code, region_code_for_country_code
-from .phonenumberutil import country_code_for_region
+from .phonenumberutil import country_code_for_region, normalize_diallable_chars_only
 from .phonenumberutil import _formatting_rule_has_first_group_only
 
 # Character used when appropriate to separate a prefix, such as a long NDD or
@@ -43,16 +43,6 @@ _EMPTY_METADATA = PhoneMetadata(id=unicod(""),
                                 international_prefix=unicod("NA"),
                                 register=False)
 
-# A pattern that is used to match character classes in regular expressions. An
-# example of a character class is [1-4].
-_CHARACTER_CLASS_PATTERN = re.compile(unicod("\\[([^\\[\\]])*\\]"))
-# Any digit in a regular expression that actually denotes a digit. For
-# example, in the regular expression 80[0-2]\d{6,10}, the first 2 digits (8
-# and 0) are standalone digits, but the rest are not.
-# Two look-aheads are needed because the number following \\d could be a
-# two-digit number, since the phone number can be as long as 15 digits.
-_STANDALONE_DIGIT_PATTERN = re.compile(unicod("\\d(?=[^,}][^,}])"))
-
 # A set of characters that, if found in a national prefix formatting rules, are an indicator to
 # us that we should separate the national prefix from the number when formatting.
 _NATIONAL_PREFIX_SEPARATORS_PATTERN = re.compile("[- ]")
@@ -62,9 +52,12 @@ _NATIONAL_PREFIX_SEPARATORS_PATTERN = re.compile("[- ]")
 # the format element under number_format contains groups of the dollar sign
 # followed by a single digit, separated by valid phone number
 # punctuation. This prevents invalid punctuation (such as the star sign in
-# Israeli star numbers) getting into the output of the AYTF.
+# Israeli star numbers) getting into the output of the AYTF. We require that
+# the first group is present in the output pattern to ensure no data is lost
+# while formatting; when we format as you type, this should always be the case.
 _ELIGIBLE_FORMAT_PATTERN = re.compile(unicod("[") + _VALID_PUNCTUATION + unicod("]*") +
-                                      unicod("(\\\\\\d") + unicod("[") + _VALID_PUNCTUATION + unicod("]*)+"))
+                                      unicod("\\\\1") + unicod("[") + _VALID_PUNCTUATION + unicod("]*") +
+                                      unicod("(\\\\\\d") + unicod("[") + _VALID_PUNCTUATION + unicod("]*)*"))
 
 # This is the minimum length of national number accrued that is required to
 # trigger the formatter. The first element of the leading_digits_pattern of each
@@ -96,7 +89,7 @@ class AsYouTypeFormatter(object):
         Arguments:
         region_code -- The region where the phone number is being entered
 
-        Return an AsYouTypeFormatter} object, which could be used to format
+        Return an AsYouTypeFormatter object, which could be used to format
         phone numbers in the specific region "as you type"
         """
         self._clear()
@@ -135,23 +128,36 @@ class AsYouTypeFormatter(object):
         return False
 
     def _get_available_formats(self, leading_digits):
-        if (self._is_complete_number and
+        # First decide whether we should use international or national number rules.
+        is_international_number = (self._is_complete_number and len(self._extracted_national_prefix) == 0)
+        if (is_international_number and
             len(self._current_metadata.intl_number_format) > 0):
             format_list = self._current_metadata.intl_number_format
         else:
             format_list = self._current_metadata.number_format
-        national_prefix_is_used_by_country = (self._current_metadata.national_prefix is not None)
         for this_format in format_list:
-            if (not national_prefix_is_used_by_country or self._is_complete_number or
-                this_format.national_prefix_optional_when_formatting or
-                _formatting_rule_has_first_group_only(this_format.national_prefix_formatting_rule)):
-                if self._is_format_eligible(this_format.format):
-                    self._possible_formats.append(this_format)
+            # Discard a few formats that we know are not relevant based on the presence of the national
+            # prefix.
+            if (len(self._extracted_national_prefix) > 0 and
+                _formatting_rule_has_first_group_only(this_format.national_prefix_formatting_rule) and
+                not this_format.national_prefix_optional_when_formatting and
+                not (this_format.domestic_carrier_code_formatting_rule is not None)):
+                # If it is a national number that had a national prefix, any rules that aren't valid with a
+                # national prefix should be excluded. A rule that has a carrier-code formatting rule is
+                # kept since the national prefix might actually be an extracted carrier code - we don't
+                # distinguish between these when extracting it in the AYTF.
+                continue
+            elif (len(self._extracted_national_prefix) == 0 and
+                  not self._is_complete_number and
+                  not _formatting_rule_has_first_group_only(this_format.national_prefix_formatting_rule) and
+                  not this_format.national_prefix_optional_when_formatting):
+                # This number was entered without a national prefix, and this formatting rule requires one,
+                # so we discard it.
+                continue
+            if fullmatch(_ELIGIBLE_FORMAT_PATTERN, this_format.format):
+                self._possible_formats.append(this_format)
         self._narrow_down_possible_formats(leading_digits)
 
-    def _is_format_eligible(self, format):
-        return fullmatch(_ELIGIBLE_FORMAT_PATTERN, format)
-
     def _narrow_down_possible_formats(self, leading_digits):
         index_of_leading_digits_pattern = len(leading_digits) - _MIN_LEADING_DIGITS_LENGTH
         ii = 0
@@ -172,17 +178,6 @@ class AsYouTypeFormatter(object):
 
     def _create_formatting_template(self, num_format):
         number_pattern = num_format.pattern
-
-        # The formatter doesn't format numbers when number_pattern contains
-        # "|", e.g.  (20|3)\d{4}. In those cases we quickly return.
-        if number_pattern.find('|') != -1:
-            return False
-
-        # Replace anything in the form of [..] with \d
-        number_pattern = re.sub(_CHARACTER_CLASS_PATTERN, unicod("\\\\d"), number_pattern)
-
-        # Replace any standalone digit (not the one in d{}) with \d
-        number_pattern = re.sub(_STANDALONE_DIGIT_PATTERN, unicod("\\\\d"), number_pattern)
         self.formatting_template = U_EMPTY_STRING
         temp_template = self._get_formatting_template(number_pattern, num_format.format)
         if len(temp_template) > 0:
@@ -364,6 +359,9 @@ class AsYouTypeFormatter(object):
         self._able_to_format = True
         self._is_expecting_country_calling_code = False
         self._possible_formats = []
+        self._last_match_position = 0
+        self._formatting_template = U_EMPTY_STRING
+        self._current_formatting_pattern = U_EMPTY_STRING
         return self._attempt_to_choose_formatting_pattern()
 
     # Some national prefixes are a substring of others. If extracting the
@@ -388,7 +386,7 @@ class AsYouTypeFormatter(object):
                  fullmatch(_PLUS_CHARS_PATTERN, next_char)))
 
     def _attempt_to_format_accrued_digits(self):
-        """Check to see if there is an exact pattern match for these digits. If so, we should use this
+        """Checks to see if there is an exact pattern match for these digits. If so, we should use this
         instead of any other formatting template whose leadingDigitsPattern also matches the input.
         """
         for number_format in self._possible_formats:
@@ -399,7 +397,18 @@ class AsYouTypeFormatter(object):
                 else:
                     self._should_add_space_after_national_prefix = bool(_NATIONAL_PREFIX_SEPARATORS_PATTERN.search(number_format.national_prefix_formatting_rule))
                 formatted_number = re.sub(num_re, number_format.format, self._national_number)
-                return self._append_national_number(formatted_number)
+                # Check that we did not remove nor add any extra digits when we matched
+                # this formatting pattern. This usually happens after we entered the last
+                # digit during AYTF. Eg: In case of MX, we swallow mobile token (1) when
+                # formatted but AYTF should retain all the number entered and not change
+                # in order to match a format (of same leading digits and length) display
+                # in that way.
+                full_output = self._append_national_number(formatted_number)
+                formatted_number_digits_only = normalize_diallable_chars_only(full_output)
+                if formatted_number_digits_only == self._accrued_input_without_formatting:
+                    # If it's the same (i.e entered number and format is same), then it's
+                    # safe to return this in formatted number as nothing is lost / added.
+                    return full_output
         return U_EMPTY_STRING
 
     def get_remembered_position(self):
@@ -534,7 +543,6 @@ class AsYouTypeFormatter(object):
         if len(self._national_number) == 0:
             return False
 
-        number_without_ccc = U_EMPTY_STRING
         country_code, number_without_ccc = _extract_country_code(self._national_number)
         if country_code == 0:
             return False
@@ -577,6 +585,9 @@ class AsYouTypeFormatter(object):
         return normalized_char
 
     def _input_digit_helper(self, next_char):
+        # Note that formattingTemplate is not guaranteed to have a value, it
+        # could be empty, e.g. when the next digit is entered after extracting
+        # an IDD or NDD.
         digit_match = _DIGIT_PATTERN.search(self._formatting_template, self._last_match_position)
         if digit_match:
             # Reset to search for _DIGIT_PLACEHOLDER from start of string
