@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 """Unit tests for shortnumberinfo.py"""
 
 # Based on original Java code:
@@ -17,12 +16,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import sys
+
 import phonenumbers
 from phonenumbers import connects_to_emergency_number, is_emergency_number, ShortNumberCost
 from phonenumbers import is_possible_short_number_for_region, is_possible_short_number
 from phonenumbers import is_valid_short_number_for_region, is_valid_short_number
-from phonenumbers import expected_cost, expected_cost_for_region
-from phonenumbers import shortnumberinfo, ShortNumberCost, PhoneNumber
+from phonenumbers import expected_cost, expected_cost_for_region, PhoneMetadata
+from phonenumbers import shortnumberinfo, ShortNumberCost, PhoneNumber, NumberParseException
 from phonenumbers.util import u
 from .testmetadatatest import TestMetadataTestCase
 
@@ -32,7 +33,7 @@ def _parse(number, regionCode):
         return phonenumbers.parse(number, regionCode)
     except NumberParseException:
         e = sys.exc_info()[1]
-        self.fail("Test input data should always parse correctly: %s (%s) => %s %s" % (number, regionCode, e))
+        raise Exception("Test input data should always parse correctly: %s (%s) => %s" % (number, regionCode, e))
 
 
 # Note that these test use real metadata for short numbers, but test metadata o/w.
@@ -51,7 +52,7 @@ class ShortNumberInfoTest(TestMetadataTestCase):
         self.assertTrue(is_possible_short_number(PhoneNumber(country_code=44, national_number=11001)))
 
         # Python version extra test: check invalid region code
-        self.assertFalse(is_possible_short_number_for_region("123456", "XY"))
+        self.assertFalse(is_possible_short_number_for_region(_parse("123456", "US"), "XY"))
 
     def testIsValidShortNumber(self):
         self.assertTrue(is_valid_short_number(PhoneNumber(country_code=33, national_number=1010)))
@@ -63,13 +64,36 @@ class ShortNumberInfoTest(TestMetadataTestCase):
         self.assertTrue(is_valid_short_number(PhoneNumber(country_code=44, national_number=18001)))
 
         # Python version extra test: check invalid region code
-        self.assertFalse(is_valid_short_number_for_region("123456", "XY"))
+        self.assertFalse(is_valid_short_number_for_region(_parse("123456", "US"), "XY"))
         self.assertFalse(is_valid_short_number(PhoneNumber(country_code=99, national_number=123)))
         # Python version extra test: not matching general desc
         self.assertFalse(is_valid_short_number_for_region(_parse("2123456", "US"), "US"))
         # Python version extra test: shared country code (44 => GB+GG) but not valid in either
         self.assertFalse(is_valid_short_number(PhoneNumber(country_code=44, national_number=58001)))
 
+    def testIsCarrierSpecific(self):
+        carrierSpecificNumber = PhoneNumber(country_code=1, national_number=33669)
+        self.assertTrue(shortnumberinfo.is_carrier_specific(carrierSpecificNumber))
+        self.assertTrue(shortnumberinfo.is_carrier_specific_for_region(_parse("33669", "US"), "US"))
+
+        notCarrierSpecificNumber = PhoneNumber(country_code=1, national_number=911)
+        self.assertFalse(shortnumberinfo.is_carrier_specific(notCarrierSpecificNumber))
+        self.assertFalse(shortnumberinfo.is_carrier_specific_for_region(_parse("911", "US"), "US"))
+
+        carrierSpecificNumberForSomeRegion = PhoneNumber(country_code=1, national_number=211)
+        self.assertTrue(shortnumberinfo.is_carrier_specific(carrierSpecificNumberForSomeRegion))
+        self.assertTrue(shortnumberinfo.is_carrier_specific_for_region(carrierSpecificNumberForSomeRegion, "US"))
+        self.assertFalse(shortnumberinfo.is_carrier_specific_for_region(carrierSpecificNumberForSomeRegion, "BB"))
+        # Python version extra test: check invalid region code
+        self.assertFalse(shortnumberinfo.is_carrier_specific_for_region(carrierSpecificNumberForSomeRegion, "XY"))
+
+    def testIsSmsService(self):
+        smsServiceNumberForSomeRegion = PhoneNumber(country_code=1, national_number=21234)
+        self.assertTrue(shortnumberinfo.is_sms_service_for_region(smsServiceNumberForSomeRegion, "US"))
+        self.assertFalse(shortnumberinfo.is_sms_service_for_region(smsServiceNumberForSomeRegion, "BB"))
+        # Python version extra test: check invalid region code
+        self.assertFalse(shortnumberinfo.is_sms_service_for_region(smsServiceNumberForSomeRegion, "XY"))
+
     def testGetExpectedCost(self):
         premiumRateExample = shortnumberinfo._example_short_number_for_cost("FR", ShortNumberCost.PREMIUM_RATE)
         self.assertEqual(ShortNumberCost.PREMIUM_RATE, expected_cost_for_region(_parse(premiumRateExample, "FR"), "FR"))
@@ -91,7 +115,7 @@ class ShortNumberInfoTest(TestMetadataTestCase):
         self.assertEqual(ShortNumberCost.UNKNOWN_COST, expected_cost(unknownCostNumber))
 
         # Test that an invalid number may nevertheless have a cost other than UNKNOWN_COST.
-        self.assertFalse(is_valid_short_number_for_region("116123", "FR"))
+        self.assertFalse(is_valid_short_number_for_region(_parse("116123", "FR"), "FR"))
         invalidNumber = PhoneNumber(country_code=33, national_number=116123)
         self.assertEqual(ShortNumberCost.TOLL_FREE, expected_cost_for_region(_parse("116123", "FR"), "FR"))
         self.assertFalse(is_valid_short_number(invalidNumber))
@@ -103,6 +127,9 @@ class ShortNumberInfoTest(TestMetadataTestCase):
         unknownCostNumber.national_number = 911
         self.assertEqual(ShortNumberCost.UNKNOWN_COST, expected_cost(unknownCostNumber))
 
+        # Python version extra test: ask for short number for invalid region
+        self.assertEqual(len(shortnumberinfo._example_short_number_for_cost("Bogus", ShortNumberCost.PREMIUM_RATE)), 0)
+
     def testGetExpectedCostForSharedCountryCallingCode(self):
         # Test some numbers which have different costs in countries sharing
         # the same country calling code. In Australia, 1234 is premium-rate,
@@ -147,24 +174,11 @@ class ShortNumberInfoTest(TestMetadataTestCase):
         self.assertEqual(ShortNumberCost.UNKNOWN_COST,
                          shortnumberinfo.expected_cost(ambiguousTollFreeNumber))
 
-    def testGetExampleShortNumber(self):
-        self.assertEqual("8711", shortnumberinfo._example_short_number("AM"))
-        self.assertEqual("1010", shortnumberinfo._example_short_number("FR"))
-        self.assertEqual("", shortnumberinfo._example_short_number("001"))
-        self.assertEqual("", shortnumberinfo._example_short_number(None))
-
-    def testGetExampleShortNumberForCost(self):
-        self.assertEqual("3010",
-                         shortnumberinfo._example_short_number_for_cost("FR", ShortNumberCost.TOLL_FREE))
-        self.assertEqual("1023",
-                         shortnumberinfo._example_short_number_for_cost("FR", ShortNumberCost.STANDARD_RATE))
-        self.assertEqual("42000",
-                         shortnumberinfo._example_short_number_for_cost("FR", ShortNumberCost.PREMIUM_RATE))
-        self.assertEqual("",
-                         shortnumberinfo._example_short_number_for_cost("FR", ShortNumberCost.UNKNOWN_COST))
-        # Python version extra test
-        self.assertEqual("",
-                         shortnumberinfo._example_short_number_for_cost("XY", ShortNumberCost.UNKNOWN_COST))
+    def testExampleShortNumberPresence(self):
+        self.assertFalse(len(shortnumberinfo._example_short_number("AD")) == 0)
+        self.assertFalse(len(shortnumberinfo._example_short_number("FR")) == 0)
+        self.assertTrue(len(shortnumberinfo._example_short_number("001")) == 0)
+        self.assertTrue(len(shortnumberinfo._example_short_number(None)) == 0)
 
     def testConnectsToEmergencyNumber_US(self):
         self.assertTrue(connects_to_emergency_number("911", "US"))
@@ -296,5 +310,16 @@ class ShortNumberInfoTest(TestMetadataTestCase):
         self.assertEqual(ShortNumberCost.UNKNOWN_COST,
                          shortnumberinfo.expected_cost_for_region(_parse("211", "US"), "US"))
         self.assertFalse(is_emergency_number("211", "CA"))
-        self.assertEqual(ShortNumberCost.UNKNOWN_COST,
+        self.assertEqual(ShortNumberCost.TOLL_FREE,
                          shortnumberinfo.expected_cost_for_region(_parse("211", "CA"), "CA"))
+
+    def testMetadataPrint(self):
+        # Python version extra test
+        # Convert all metadata to strings to check the printing code doesn't blow up.
+        for region_code in PhoneMetadata._short_region_available.keys():
+            metadata = PhoneMetadata.short_metadata_for_region(region_code)
+            str(metadata)
+
+    def testMetadataAbsent(self):
+        # Python version extra test: check internal fn. copes with missing PhoneNumberDesc
+        self.assertFalse(shortnumberinfo._match_national_number("123456", None, False))
